@@ -7,7 +7,7 @@ use warnings;
 use List::Util qw{ first };
 use List::MoreUtils qw{ true first_index };
 
-our $VERSION = '0.35';
+our $VERSION = '0.40';
 
 
 
@@ -24,7 +24,7 @@ my %role_type = (
 sub new {
     my $class = shift;
 
-    our $self = $class->SUPER::new( shift );
+    my $self = $class->SUPER::new( shift );
 
     $self->{latlon}         = {};
     $self->{waychain}       = {};
@@ -33,19 +33,29 @@ sub new {
 
     our %param = @_;
 
-    ## First pass - load multipolygon parameters
+    
+    
+    ## First pass - load nodes and multipolygon parameters
 
     my $osm_pass1 = sub {
         my ($obj) = @_;
 
-        if ( $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
-            # old-style multipolygons - load lists of inner rings
+        # node
+        if ( $obj->{type} eq 'node' ) {
+            $self->{latlon}->{ $obj->{id} } = pack 'Z*Z*', $obj->{lat}, $obj->{lon};
+        }
+
+        #multipolygon
+        if ( $obj->{type} eq 'relation' 
+                && exists $obj->{tag}->{type}
+                && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
+            # old-style - list of inner rings
             if ( (true { $_->{role} eq 'outer' } @{ $obj->{members} }) == 1 ) {
                 my $outer = first { $_->{role} eq 'outer' } @{ $obj->{members} };
                 $self->{mpoly}->{$outer->{ref}} = 
                     [ map { $_->{ref} } grep { $_->{role} eq 'inner' } @{ $obj->{members} } ];
             }
-            # advanced multipolygons - load lists of ways
+            # advanced - ways
             for my $member ( @{ $obj->{members} } ) {
                 next unless $member->{type} eq 'way';
                 next unless exists $role_type{ $member->{role} };
@@ -56,29 +66,30 @@ sub new {
         &{ $param{pass1} }( $obj )  if exists $param{pass1};
     };
 
-    $self->SUPER::parse( $osm_pass1, only => 'relation' );
+    $self->SUPER::parse( $osm_pass1, only => [ 'node', 'relation' ] );
 
+
+    
     ## Second pass - load necessary primitives
 
     $self->seek_to(0);
+    # $self->seek_to_ways();
 
-    &{ $param{between} }()  if exists $param{between};
+    &{ $param{between} }( $self )  if exists $param{between};
 
     my $osm_pass2 = sub {
-        my ($obj) = @_;
+        my ($obj, $self) = @_;
 
-        if ( $obj->{type} eq 'node' ) {
-            $self->{latlon}->{ $obj->{id} } = pack 'Z*Z*', $obj->{lat}, $obj->{lon};
-        }
-        elsif ( $obj->{type} eq 'way' && exists $self->{ways_to_load}->{$obj->{id}} ) {
+        if ( $obj->{type} eq 'way' && exists $self->{ways_to_load}->{$obj->{id}} ) {
             $self->{waychain}->{ $obj->{id} } = $obj->{chain};
             delete $self->{ways_to_load}->{$obj->{id}};
         }
 
-        &{ $param{pass2} }( $obj )  if exists $param{pass2};
+        &{ $param{pass2} }( $obj, $self )  if exists $param{pass2};
     };
 
-    $self->SUPER::parse( $osm_pass2 );
+    #$self->SUPER::parse( $osm_pass2 );
+    $self->parse( $osm_pass2 );
 
     $self->seek_to(0);
 
@@ -89,26 +100,27 @@ sub new {
 
 sub parse {
 
-    our $self = shift;
+    my $self = shift;
     our $callback = shift;
 
     my $parse_extent = sub {
         my ($obj) = @_;
 
-        # old-style multipolygons
-        if ( $obj->{type} eq 'way' && exists $self->{mpoly}->{ $obj->{id} }
-                && $obj->{chain}->[0] eq $obj->{chain}->[-1] ) {
-            $obj->{outer} = [ [ @{$obj->{chain}} ] ];
-            # $obj->{outer} = [ $obj->{chain} ];
-            for my $inner ( @{ $self->{mpoly}->{ $obj->{id} } } ) {
-                next unless exists $self->{waychain}->{$inner};
-                next unless $self->{waychain}->{$inner}->[0] eq $self->{waychain}->{$inner}->[-1];
-                push @{ $obj->{inner} }, $self->{waychain}->{$inner};
+        # closed ways and old-style multipolygons
+        if ( $obj->{type} eq 'way' && $obj->{chain}->[0] eq $obj->{chain}->[-1] ) {
+            # $obj->{outer} = [ [ @{$obj->{chain}} ] ];
+            $obj->{outer} = [ $obj->{chain} ];
+            if ( exists $self->{mpoly}->{ $obj->{id} } ) {
+                for my $inner ( @{ $self->{mpoly}->{ $obj->{id} } } ) {
+                    next unless exists $self->{waychain}->{$inner};
+                    next unless $self->{waychain}->{$inner}->[0] eq $self->{waychain}->{$inner}->[-1];
+                    push @{ $obj->{inner} }, $self->{waychain}->{$inner};
+                }
             }
         }
 
         # advanced multipolygons
-        if ( $obj->{type} eq 'relation' && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
+        if ( $obj->{type} eq 'relation' && exists $obj->{tag}->{type} && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
 
             for my $contour_type ( 'outer', 'inner' ) {
     
@@ -158,7 +170,7 @@ sub parse {
             } # outers/inners
         } # advanced multipolygon
 
-        &$callback( @_ );
+        &$callback( $obj, $self );
     };
     
     $self->SUPER::parse( $parse_extent, @_ );    
@@ -199,14 +211,14 @@ Geo::Parse::OSM::Multipass extends Geo::Parse::OSM class to resolve geometry.
     my $osm = Geo::Parse::OSM::Multipass->new( 'planet.osm' );
 
 Creates parser instance and makes two passes:
-1 - only relations, create the list of multipolygon parts
-2 - load those parts and nodes
+1 - load node coordinates and create list of multipolygon parts
+2 - load those parts
 
 You can add extra custom callback function:
 
     my $osm = Geo::Parse::OSM::Multipass->new( 'planet.osm', pass1 => sub{ ... } );
 
-* pass1 - for every object during 1st pass
+* pass1 - is called for every object during 1st pass
 * pass2 - same for second pass
 * between - before 2nd pass
 
